/**
 *
 * @author yutent<yutent.io@gmail.com>
 * @date 2019/09/16 20:51:19
 */

const { app, BrowserWindow, protocol, Menu, ipcMain } = require('electron')
const { join, resolve, extname } = require('path')
const { cat, echo, isfile } = require('./tools/utils.js')
const os = require('os')
const createMenu = require('./tools/menu')

const MIME_TYPES = {
  '.js': 'text/javascript',
  '.html': 'text/html',
  '.htm': 'text/plain',
  '.css': 'text/css',
  '.jpg': 'image/jpg',
  '.png': 'image/png',
  '.gif': 'image/gif',
  '.svg': 'image/svg+xml',
  '.ico': 'image/ico'
}

const ROOT = __dirname
const HOST_FILE = resolve(app.getPath('userData'), 'host.cache')
const LOCK_FILE = resolve(app.getPath('userData'), 'lock')

/* ********** 修复环境变量 start *********** */
let pathEnv = new Set()
process.env.PATH.split(':').forEach(_ => {
  pathEnv.add(_)
})
pathEnv.add('/usr/local/bin')
pathEnv.add('/usr/local/sbin')

process.env.PATH = Array.from(pathEnv).join(':')
pathEnv = null

/* ********** 修复环境变量 end *********** */

/* ----------------------------------------------------- */
app.commandLine.appendSwitch('--lang', 'zh-CN')
app.commandLine.appendSwitch('--autoplay-policy', 'no-user-gesture-required')

protocol.registerSchemesAsPrivileged([
  { scheme: 'app', privileges: { secure: true, standard: true } }
])

/* ----------------------------------------------------- */

if (!isfile(HOST_FILE)) {
  echo('{}', HOST_FILE)
}

ipcMain.on('ready', ev => {
  ev.returnValue = { HOST_FILE, LOCK_FILE, HOST: os.hostname() }
})

//  初始化应用
app.once('ready', () => {
  // 注册协议
  protocol.registerBufferProtocol('app', (req, cb) => {
    let file = req.url.replace(/^app:\/\/local\//, '')
    let ext = extname(req.url)
    let buff = cat(resolve(ROOT, file))
    cb({ data: buff, mimeType: MIME_TYPES[ext] })
  })

  Menu.setApplicationMenu(null)

  // 创建浏览器窗口
  let win = new BrowserWindow({
    title: 'Hosts切换器',
    width: 1000,
    height: 640,
    resizable: false,
    maximizable: false,
    icon: resolve(ROOT, './images/app.png'),
    webPreferences: {
      // webSecurity: false,
      // experimentalFeatures: true,
      // nodeIntegration: true,
      preload: join(ROOT, './tools/inject.js')
    }
  })

  win.on('closed', () => {
    app.exit()
    win = null
  })

  // win.openDevTools()

  // 然后加载应用的 index.html
  win.loadURL('app://local/index.html')

  if (process.platform === 'darwin') {
    createMenu(win)
  }
})
