/**
 * {}
 * @author yutent<yutent.io@gmail.com>
 * @date 2022/07/14 11:54:32
 */

const { ipcRenderer, shell, contextBridge } = require('electron')
const { resolve } = require('path')
const { cat, echo, isfile, writable } = require('./utils.js')

const { HOST_FILE, LOCK_FILE, HOST } = ipcRenderer.sendSync('ready')

let timer = null

contextBridge.exposeInMainWorld('electron', {
  hostname: HOST,
  open(url) {
    shell.openExternal(url)
  },
  checkPermission() {
    return writable('/etc/hosts')
  },
  getHistory() {
    if (isfile(LOCK_FILE)) {
      var cache = cat(HOST_FILE)
      return JSON.parse(cache)
    }

    var cache = cat('/etc/hosts').toString()
    var records = cache.split(/[\n\r]+/)
    var list = []
    var dict = {}
    records.forEach(str => {
      str = str.trim()
      let matches = str.match(/^(#*?)\s*(\d+\.\d+\.\d+\.\d+)\s+(.*)/)

      if (matches) {
        let names = matches[3].split(/\s+/).map(it => it.trim())
        let name
        while ((name = names.pop())) {
          list.push({ ip: matches[2], enabled: !matches[1], name })
        }
      }
    })
    records = null

    list.forEach(it => {
      it.name = it.name.split('.')
      let domain = it.name.splice(-2, 2).join('.')
      if (domain === 'com.cn' || domain === 'org.cn' || domain === 'net.cn') {
        domain = it.name.pop() + '.' + domain
      }

      if (dict[domain]) {
        dict[domain].push({
          value: it.ip,
          enabled: it.enabled,
          record: it.name.join('.') || '@',
          remark: ''
        })
      } else {
        dict[domain] = [
          {
            value: it.ip,
            enabled: it.enabled,
            record: it.name.join('.') || '@',
            remark: ''
          }
        ]
      }
    })
    list = null
    echo(JSON.stringify(dict), HOST_FILE)
    echo('', LOCK_FILE)
    return dict
  },

  saveHosts(dict) {
    clearTimeout(timer)
    timer = setTimeout(() => {
      var txt = ''
      for (let k in dict) {
        for (let it of dict[k]) {
          if (it.enabled) {
            var name = it.record === '@' ? '' : it.record
            if (name) {
              name += '.'
            }
            txt += `${it.value.padEnd(15, ' ')}    ${name + k}\n`
          }
        }
        txt += '\n'
      }
      echo(JSON.stringify(dict), HOST_FILE)
      echo(txt, '/etc/hosts')
    }, 1000)
  }
})
