/**
 * {}
 * @author yutent<yutent.io@gmail.com>
 * @date 2023/08/08 18:19:17
 */
import { html, css, Component, classMap, nextTick } from 'wkit'

import 'ui/icon/index.js'
import 'ui/scroll/index.js'
import 'ui/layer/index.js'
import 'ui/space/index.js'
import 'ui/form/input.js'
import 'ui/form/switch.js'
import 'ui/form/button.js'
import 'ui/form/link.js'

import './permission.js'

import { getHistory, saveHosts } from '../utils/index.js'

const HOST_DATA = await getHistory()

let tmp_records = Object.create(null)

function noop() {}

class Home extends Component {
  static props = {
    filter: '',
    activeDomain: localStorage.getItem('last_domain') || '', //当前选中的域名
    editDomain: '', // 当前临时要编辑的域名, 即右键菜单选择到的
    domains: Object.keys(HOST_DATA),
    records: [],
    permissionShow: false
  }

  static styles = [
    css`
      :host {
        flex: 1;
        display: flex;
        width: 100%;
        height: 100%;
        padding: 32px;
        color: var(--color-dark-1);
        background: #f0f0f0;
      }
      .visible {
        display: block;
      }

      ul li {
        list-style: none;
      }

      .noselect {
        -webkit-touch-callout: none;
        -webkit-user-select: none;
        user-select: none;
      }
    `,
    css`
      .layout-left {
        flex-shrink: 0;
        display: flex;
        flex-direction: column;
        width: 180px;
        height: 100vh;
        background: var(--color-plain-2);
      }

      wc-scroll {
        flex: 1;
      }

      .layout-left .domain-list {
        width: 100%;
      }

      .layout-left .domain-list .item {
        display: flex;
        justify-content: flex-end;
        align-items: center;
        height: 40px;
        padding: 0 12px;
        cursor: pointer;
        transition: background 0.15s ease-in-out;
      }

      .layout-left .item wc-icon {
        --wc-icon-size: 12px;
        margin-left: 8px;
        color: var(--color-grey-2);
      }

      .layout-left .item:hover,
      .layout-left .item.active {
        background: #fff;
      }
      .layout-left .item.active {
        border-right: 2px solid var(--color-orange-1);
        color: var(--color-orange-1);
      }
      .layout-left .item.active wc-icon {
        color: var(--color-orange-1);
      }
      .layout-left .item.blank {
        justify-content: center;
        cursor: default;
      }

      .layout-left .item.blank:hover {
        background: none;
      }

      .action {
        display: flex;
        align-items: center;
        height: 50px;
        padding: 0 10px;
      }
    `,
    css`
      .layout-right {
        flex: 1;
        display: flex;
        flex-direction: column;
      }
      .layout-right .toolbar {
        display: flex;
        align-items: center;
        justify-content: space-between;
        height: 40px;
        padding: 0 15px;
        border-bottom: 1px solid var(--color-plain-3);
        background: var(--color-plain-1);
      }
    `,
    css`
      .layout-right .main {
        overflow: hidden;
        flex: 1;
        display: flex;
        flex-direction: column;
        padding: 8px;
        background: #fff;
      }
      .layout-right .item {
        display: flex;
        align-items: center;
        justify-content: center;
        height: 40px;
        padding: 0 4px;
        border-bottom: 1px solid var(--color-plain-2);
        text-align: center;
      }
      .layout-right .thead {
        line-height: 40px;
      }
      .layout-right .item span {
        flex: 1;
      }
      .layout-right .item .long {
        flex: 1.5;
      }
      .layout-right wc-scroll {
        overflow: hidden;
      }
      .layout-right .records {
        flex: 1;
      }

      .layout-right .item wc-input {
        flex: 1;
        min-width: 0;
        --wc-input-border-color: transparent;
      }
      .layout-right .record-list .item .long {
        flex: 1.5;
      }
      .layout-right .record-list .item wc-button {
        margin-left: 5px;
      }
      .layout-right .record-list .item:last-child {
        border-bottom: none;
      }
    `,

    css`
      .context-menu {
        display: flex;
        flex-direction: column;
        width: 100px;
        padding: 5px 0;
        background: #fff;
      }

      .context-menu .item {
        height: 30px;
        line-height: 30px;
        padding: 0 15px;
        cursor: pointer;
      }

      .context-menu .item :hover {
        background: #f2f5fc;
      }
    `
  ]

  mounted() {
    window.foo = this
    // let hosts = getHistory()
    // console.log(hosts)
    // this.domains = Object.keys(hosts)
    this.toggleDomain(null, this.activeDomain)
  }

  addDomain() {
    layer
      .prompt('请输入根域名', function (val, done) {
        if (
          val === 'localhost' ||
          val === 'local' ||
          /^[\w.\-]+\.[a-z]+$/.test(val)
        ) {
          done()
        } else {
          layer.toast('域名格式错误', 'error')
        }
      })
      .then(val => {
        this.domains.push(val)
        HOST_DATA[val] = []
        if (!this.activeDomain) {
          this.toggleDomain(null, val)
        }
        this.save()
      })
      .catch(noop)
  }

  toggleDomain(ev, name) {
    name = name || ev.currentTarget.dataset.name
    this.activeDomain = name

    this.records = (HOST_DATA[name] || []).sort((a, b) =>
      a.record.localeCompare(b.record)
    )

    tmp_records = Object.create(null)
    for (let it of this.records) {
      if (tmp_records[it.record]) {
        tmp_records[it.record].push(it)
      } else {
        tmp_records[it.record] = [it]
      }
    }
    document.title = `伪域名解析   ${name} `
    localStorage.setItem('last_domain', name)
    nextTick(() => {
      this.$refs.records.scrollTop = 0
    })
  }

  showMenu(ev) {
    this.$refs.context.close()
    var { pageX, pageY } = ev
    if (pageY + 70 > 600) {
      pageY -= 70
    }

    var elem = ev.target

    if (elem.tagName !== 'LI') {
      elem = elem.parentNode
    }
    this.editDomain = elem.dataset.name
    nextTick(_ => {
      this.$refs.context.moveTo({ left: pageX + 'px', top: pageY + 'px' })
      this.$refs.context.show()
    })
  }

  confirmAction(ev) {
    this.$refs.context.close()
    if (ev.target.tagName === 'LI') {
      var act = ev.target.dataset.act

      if (act === 'del') {
        layer
          .confirm(`是否要删除域名「${this.editDomain}」?`)
          .then(res => {
            if (this.editDomain === this.activeDomain) {
              if (this.records.length) {
                return layer.toast(
                  '该域名下有主机记录, 请先删除主机记录后再删除域名',
                  'error'
                )
              }
            } else {
              if (HOST_DATA[this.editDomain].length > 0) {
                return layer.toast(
                  '该域名下有主机记录, 请先删除主机记录后再删除域名',
                  'error'
                )
              }
            }
            delete HOST_DATA[this.editDomain]
            this.domains.remove(this.editDomain)
            this.activeDomain = ''
            this.editDomain = ''
            this.records.clear()
            this.save()
            this.toggleDomain(this.domains[0])
          })
          .catch(noop)
      } else if (act === 'edit') {
        layer
          .prompt(`请输入新的名字「${this.editDomain}」`, (val, done) => {
            if (val === this.editDomain || HOST_DATA[val]) {
              return
            }

            if (
              val === 'localhost' ||
              val === 'local' ||
              /^[\w.]+\.[a-z]+$/.test(val)
            ) {
              done()
            } else {
              layer.toast('域名格式错误', 'error')
            }
          })
          .then(val => {
            var idx = this.domains.indexOf(this.editDomain)
            this.domains.set(idx, val)
            HOST_DATA[val] = HOST_DATA[this.editDomain]
            delete HOST_DATA[this.editDomain]
            this.activeDomain = ''
            this.editDomain = ''
            this.save()
            this.toggleDomain(val)
          })
          .catch(noop)
      }
    }
  }

  // 同一个记录, 允许一条被激活
  recordChanges(item) {
    if (item.enabled) {
      if (tmp_records[item.record].length > 1) {
        for (let it of tmp_records[item.record]) {
          if (it.value !== item.value) {
            it.enabled = false
          }
        }
      }
    }
  }

  save() {
    if (this.activeDomain) {
      HOST_DATA[this.activeDomain] = this.records
    }
    // electron.saveHosts(HOST_DATA)
    layer.toast('保存成功', 'success')
  }

  render() {
    return html`
      <div class="layout-left noselect">
        <wc-scroll>
          <ul class="domain-list" @contextmenu.prevent=${this.showMenu}>
            ${this.domains.map(
              it => html`
                <li
                  class=${classMap({
                    item: true,
                    active: it === this.activeDomain
                  })}
                  data-name=${it}
                  @click=${this.toggleDomain}
                >
                  <span>${it}</span>
                  <wc-icon name="right"></wc-icon>
                </li>
              `
            )}
            ${this.domains.length < 1
              ? html`<li class="item blank">没有域名</li>`
              : ''}
          </ul>
        </wc-scroll>
        <section class="action">
          <wc-button circle icon="plus" @click=${this.addDomain}> </wc-button>
        </section>
      </div>

      <div class="layout-right noselect">
        <header class="toolbar">
          <wc-button size="m" icon="plus" type="primary" @click="addRecord"
            >新增记录</wc-button
          >
          <wc-button size="m" icon="fly" type="info" @click="save"
            >保存</wc-button
          >
        </header>

        <main class="main">
          <header class="thead item">
            <span>主机记录</span>
            <span>类型</span>
            <span class="long">记录值</span>
            <span>操作</span>
            <span>备注</span>
          </header>
          <wc-scroll class="records" ref="records">
            ${this.records.map(
              it => html`
                <div class="item">
                  <wc-input
                    size="m"
                    autofocus
                    @change="updateCacheDict(it)"
                    :value=${it.record}
                    label="根域请填 @"
                  ></wc-input>
                  <span>A</span>
                  <wc-input
                    size="m"
                    class="long"
                    :value=${it.value}
                    label="请填写IP"
                  ></wc-input>
                  <wc-space gap="s">
                    <wc-link size="m" type="danger" @click="$remove"
                      >删除</wc-link
                    >
                    <wc-link size="m" type="info" @click="clone(it)"
                      >克隆</wc-link
                    >
                    <wc-switch
                      size="m"
                      :value=${it.enabled}
                      @change=${this.recordChanges(it)}
                    ></wc-switch>
                  </wc-space>
                  <wc-input size="m" no-border :value=${it.remark}></wc-input>
                </div>
              `
            )}
          </wc-scroll>
        </main>
      </div>

      <wc-permission
        class=${classMap({ visible: this.permissionShow })}
      ></wc-permission>

      <wc-layer ref="context" left="100px" top="0" radius="0">
        <ul class="context-menu" @click=${this.confirmAction}>
          <li class="item" data-act="del">删除域名</li>
          <li class="item" data-act="edit">修改域名</li>
        </ul>
      </wc-layer>
    `
  }
}

Home.reg('home')
