const CONFIG_DIR = '/home/yutent/.config/hosts-switch'
const HOST_FILE = `${CONFIG_DIR}/host.cache`
const LOCK_FILE = `${CONFIG_DIR}/lock`

let timer

export function checkPermission() {
  return writable('/etc/hosts')
}

export async function getHistory() {
  if (await native.fs.isfile(LOCK_FILE)) {
    var cache = await native.fs.read(HOST_FILE)
    return JSON.parse(cache)
  }

  var cache = native.fs.read('/etc/hosts').toString()
  var records = cache.split(/[\n\r]+/)
  var list = []
  var dict = {}
  records.forEach(str => {
    str = str.trim()
    let matches = str.match(/^(#*?)\s*(\d+\.\d+\.\d+\.\d+)\s+(.*)/)

    if (matches) {
      let names = matches[3].split(/\s+/).map(it => it.trim())
      let name
      while ((name = names.pop())) {
        list.push({ ip: matches[2], enabled: !matches[1], name })
      }
    }
  })
  records = null

  list.forEach(it => {
    it.name = it.name.split('.')
    let domain = it.name.splice(-2, 2).join('.')
    if (domain === 'com.cn' || domain === 'org.cn' || domain === 'net.cn') {
      domain = it.name.pop() + '.' + domain
    }

    if (dict[domain]) {
      dict[domain].push({
        value: it.ip,
        enabled: it.enabled,
        record: it.name.join('.') || '@',
        remark: ''
      })
    } else {
      dict[domain] = [
        {
          value: it.ip,
          enabled: it.enabled,
          record: it.name.join('.') || '@',
          remark: ''
        }
      ]
    }
  })
  list = null
  native.fs.write(HOST_FILE, JSON.stringify(dict))
  native.fs.write(LOCK_FILE, '')
  return dict
}

export function saveHosts(dict) {
  clearTimeout(timer)
  timer = setTimeout(() => {
    var txt = ''
    for (let k in dict) {
      for (let it of dict[k]) {
        if (it.enabled) {
          var name = it.record === '@' ? '' : it.record
          if (name) {
            name += '.'
          }
          txt += `${it.value.padEnd(15, ' ')}    ${name + k}\n`
        }
      }
      txt += '\n'
    }
    native.fs.write(HOST_FILE, JSON.stringify(dict))
    native.fs.write('/etc/hosts', txt)
  }, 1000)
}
